@@ -46,7 +46,8 @@ class AutoTraderBot():
     
     '''
     
-    def __init__(self, instrument, strategy_config, broker, data_dict, autotrader_instance):
+    def __init__(self, instrument, strategy_config, broker, data_dict, 
+                 quote_data_dict, auxdata, autotrader_instance):
         '''
         AutoTrader Bot initialisation. 
         '''
@@ -142,6 +143,7 @@ class AutoTraderBot():
             self.MTF_data = None
         
         # Data retrieval
+        self.quote_data_file = quote_data_dict
         if data_dict is not None:
             # Local data files provided
             self.abs_data_filepath = True
@@ -163,6 +165,11 @@ class AutoTraderBot():
         else:
             strat_data = MTF_data
         
+        # Auxiliary data assignment
+        if auxdata is not None:
+            strat_data = {'base': strat_data,
+                          'aux': auxdata}
+        
         # Instantiate Strategy
         include_broker = strategy_config['INCLUDE_BROKER'] if 'INCLUDE_BROKER' in strategy_config else False
         if include_broker:
@@ -279,7 +286,17 @@ class AutoTraderBot():
                 
                 # Adjust for start and end dates
                 data = self._check_data_period(data, from_date, to_date)
-                quote_data = data
+                
+                # Load quote data
+                if self.quote_data_file is not None:
+                    quote_data = pd.read_csv(self.quote_data_file, index_col = 0)
+                    quote_data.index = pd.to_datetime(quote_data.index, utc=True)
+                    quote_data = self._check_data_period(quote_data, from_date, to_date)
+                else:
+                    quote_data = data
+                
+                # Correct any data mismatches
+                data, quote_data = self._match_quote_data(data, quote_data)
                 
                 MTF_data = None
             
@@ -302,7 +319,14 @@ class AutoTraderBot():
                     data = self._check_data_period(data, from_date, to_date)
                     
                     if granularity == MTF_granularities[0]:
-                        quote_data = data
+                        # Assign quote data for backtesting
+                        if self.quote_data_file is not None:
+                            quote_data = pd.read_csv(self.quote_data_file, index_col = 0)
+                            quote_data.index = pd.to_datetime(quote_data.index, utc=True)
+                            quote_data = self._check_data_period(quote_data, from_date, to_date)
+                            data, quote_data = self._match_quote_data(data, quote_data)
+                        else:
+                            quote_data = data
                     
                     # Add to MTF_data dict
                     MTF_data[granularity] = data
@@ -939,4 +963,26 @@ class AutoTraderBot():
         
         self.data = data
         self.strategy.data = data
-    
\ No newline at end of file
+    
+    def _match_quote_data(self, data, quote_data):
+        ''' Function to match index of trading data and quote data. '''
+        datasets = [data, quote_data]
+        adjusted_datasets = []
+        
+        for dataset in datasets:
+            # Initialise common index
+            common_index = dataset.index
+            
+            # Update common index by intersection with other data 
+            for other_dataset in datasets:
+                common_index = common_index.intersection(other_dataset.index)
+            
+            # Adjust data using common index found
+            adj_data = dataset[dataset.index.isin(common_index)]
+            
+            adjusted_datasets.append(adj_data)
+        
+        # Unpack adjusted datasets
+        adj_data, adj_quote_data = adjusted_datasets
+        
+        return adj_data, adj_quote_data
\ No newline at end of file
