@@ -87,6 +87,7 @@ class AutoPlot():
         
         # Modify data index
         self.data               = self._reindex_data(data)
+        self.backtest_data      = None
         
         # Load JavaScript code for auto-scaling 
         with open(os.path.join(os.path.dirname(__file__), 'lib/autoscale.js'),
@@ -133,6 +134,35 @@ class AutoPlot():
         
         return data
         
+    def _merge_data(self, data, name=None):
+        '''
+        Merges the provided data with the base data, using the data of the 
+        base data and the index of the data to be merged.
+        
+        Parameters:
+            data: the data to be merged
+            name: the desired column name of the merged data
+        '''
+        
+        # TODO - need to add handling of different data types, ie. list vs. series vs. df
+        
+        merged_data = pd.merge(self.data, data, left_on='date', 
+                               right_index=True).fillna('')
+        
+        if name is not None:
+            merged_data.rename(columns={data.name: name}, inplace=True)
+        
+        return merged_data
+    
+    def _add_backtest_price_data(self, backtest_price_data):
+        ''' 
+        Processes backtest price data to included integer index of base 
+        data.
+        '''
+        temp_data = self.data.copy()
+        temp_data.index = temp_data['date']
+        
+        self.backtest_data = temp_data.reindex(backtest_price_data.index, method='ffill')
     
     ''' ------------------- FIGURE MANAGEMENT METHODS --------------------- '''
     def plot(self, backtest_dict=None, cumulative_PL=None, indicators=None, 
@@ -174,8 +204,8 @@ class AutoPlot():
         
         if backtest_dict is not None:
             
-            NAV             = backtest_dict['NAV']
-            balance         = backtest_dict['balance']
+            NAV             = backtest_dict['account_history']['NAV']
+            balance         = backtest_dict['account_history']['balance']
             trade_summary   = backtest_dict['trade_summary']
             indicators      = backtest_dict['indicators']
             open_trades     = backtest_dict['open_trades']
@@ -185,13 +215,9 @@ class AutoPlot():
             top_fig = self._plot_line(NAV, candle_plot, new_fig=True, 
                                       legend_label='Net Asset Value', 
                                       hover_name='NAV')
-            self._plot_line(balance, 
-                            top_fig, 
-                            legend_label='Account Balance', 
-                            hover_name='P/L',
-                            line_colour='blue')
+            self._plot_line(balance, top_fig, legend_label='Account Balance', 
+                            hover_name='P/L', line_colour='blue')
             # if cumulative_PL is not None:
-                
             #     self._plot_line(cumulative_PL, top_fig, 
             #                     legend_label='Cumulative P/L', 
             #                     hover_name='P/L')
@@ -530,7 +556,8 @@ class AutoPlot():
                      'multi'       : 'below',
                      'signals'     : 'over',
                      'bands'       : 'over',
-                     'threshold'   : 'below'}
+                     'threshold'   : 'below',
+                     'trading-session': 'over'}
         
         # Plot indicators
         indis_over              = 0
@@ -570,7 +597,11 @@ class AutoPlot():
                         self._plot_bands(indicators[indicator], 
                                          linked_fig=linked_fig, new_fig=False,
                                          legend_label=indicator)
-                        
+                    
+                    elif indi_type == 'trading-session':
+                        self._plot_trading_session(indicators[indicator],
+                                                   linked_fig)
+                    
                     else:
                         # Generic overlay indicator - plot as line
                         if type(indicators[indicator]['data']) == pd.Series:
@@ -717,8 +748,14 @@ class AutoPlot():
             fig = linked_fig
         
         # Add glyphs
-        source = ColumnDataSource(self.data)
-        source.add(plot_data, 'plot_data')
+        if len(plot_data) != len(self.data):
+            # Mismatched timeframe
+            merged_data = self._merge_data(plot_data, name='plot_data')
+            source = ColumnDataSource(merged_data)
+        else:
+            source = ColumnDataSource(self.data)
+            source.add(plot_data, 'plot_data')
+        
         fig.line('data_index', 'plot_data', 
                  line_color         = line_colour,
                  legend_label       = legend_label,
@@ -934,6 +971,39 @@ class AutoPlot():
                                        line_color = 'red',
                                        legend_label = 'Resistance 3')
         
+    def _plot_trading_session(self, session_plot_data, linked_fig):
+        'Shades trading session times'
+        
+        session = session_plot_data['data'].lower()
+        fill_color = session_plot_data['fill_color'] if 'fill_color' in session_plot_data else 'blue'
+        fill_alpha = session_plot_data['fill_alpha'] if 'fill_alpha' in session_plot_data else 0.3
+        line_color = session_plot_data['line_color'] if 'line_color' in session_plot_data else None
+        
+        times = {'sydney': {'start': '21:00', 'end': '05:00'},
+                 'london': {'start': '08:00', 'end': '16:00'},
+                 'new york': {'start': '13:00', 'end': '21:00'},
+                 'tokyo': {'start': '23:00', 'end': '07:00'},
+                 'frankfurt': {'start': '07:00', 'end': '15:00'}
+                 }
+        
+        index_data = self.data.set_index('date')
+        
+        midpoint = max(self.data.High.values)
+        height = 2*midpoint
+        
+        session_start = times[session]['start']
+        session_end = times[session]['end']
+        
+        session_data = index_data.between_time(session_start, session_end)
+        
+        opens = session_data[session_data.data_index - session_data.data_index.shift(1) != 1].data_index
+        closes = session_data[(session_data.data_index - session_data.data_index.shift(1) != 1).shift(-1).fillna(True)].data_index
+        
+        linked_fig.hbar(midpoint, height, opens, closes, 
+                        line_color = line_color, 
+                        fill_color = fill_color,
+                        fill_alpha = fill_alpha,
+                        legend_label = f'{session} trading session')
         
         
     ''' ----------------------- TOP FIG PLOTTING -------------------------- '''
@@ -954,13 +1024,15 @@ class AutoPlot():
                             cancelled_summary=False, open_summary=False):
         ''' Plots trades taken over ohlc chart. '''
         
-        ts = trade_summary
-        # TODO - merge should work with left_on='date', right_index=True,
-        # meaning this can be deleted below - test it
-        ts['date']   = ts.index 
-        ts           = ts.reset_index(drop = True)
+        exit_summary = trade_summary.copy()
         
-        trade_summary = pd.merge(self.data, ts, left_on='date', right_on='date')
+        if self.backtest_data is not None:
+            # Charting on different timeframe data
+            trade_summary = pd.merge(self.backtest_data, trade_summary, 
+                                     left_index=True, right_index=True)
+        else:
+            trade_summary = pd.merge(self.data, trade_summary, 
+                                     left_on='date', right_index=True)
         
         # Backtesting signals
         long_trades             = trade_summary[trade_summary.Size > 0]
@@ -968,7 +1040,13 @@ class AutoPlot():
         
         if cancelled_summary is False and open_summary is False:
             
-            exit_summary = pd.merge(self.data, ts, left_on='date', right_on='Exit_time')
+            if self.backtest_data is not None:
+                # Charting on different timeframe data
+                exit_summary = pd.merge(self.backtest_data, exit_summary, 
+                                        left_index=True, right_on='Exit_time')
+            else:
+                exit_summary = pd.merge(self.data, exit_summary, 
+                                        left_on='date', right_on='Exit_time')
             
             profitable_longs        = long_trades[(long_trades['Profit'] > 0)]
             unprofitable_longs      = long_trades[(long_trades['Profit'] < 0)]
@@ -1036,13 +1114,13 @@ class AutoPlot():
         # Stop loss  levels
         if None not in trade_summary.Stop_loss.values:
             self._plot_trade(list(trade_summary.data_index.values),
-                             list(trade_summary.Stop_loss.values),
+                             list(trade_summary.Stop_loss.fillna('').values),
                              'dash', 'black', 'Stop loss', linked_fig)
         
         # Take profit levels
         if None not in trade_summary.Take_profit.values:
             self._plot_trade(list(trade_summary.data_index.values),
-                             list(trade_summary.Take_profit.values),
+                             list(trade_summary.Take_profit.fillna('').values),
                              'dash', 'black', 'Take profit', linked_fig)
         
         # Position exits
@@ -1175,18 +1253,24 @@ class AutoPlot():
         else:
             # Plot over linked figure
             fig = linked_fig
-            
-        fig.varea(self.data.index, 
-                  plot_data['lower'], 
-                  plot_data['upper'],
+        
+        # Charting on different timeframe data
+        lower_band = self._merge_data(plot_data['lower'], name='lower')['lower']
+        upper_band = self._merge_data(plot_data['upper'], name='upper')['upper']
+        
+        fig.varea(lower_band.index, 
+                  lower_band.values, 
+                  upper_band.values,
                   fill_alpha = fill_alpha, 
                   fill_color = fill_color,
                   legend_label = plot_data['band_name'] if 'band_name' in plot_data else legend_label)
         
         if 'mid' in plot_data:
             # Add a mid line
-            fig.line(self.data.index, plot_data['mid'], line_color=line_color,
+            mid_line = self._merge_data(plot_data['mid'], name='mid')['mid']
+            fig.line(mid_line.index, mid_line.values, line_color=line_color,
             legend_label = plot_data['mid_name'] if 'mid_name' in plot_data else 'Band Mid Line')
         
         return fig
+    
     
\ No newline at end of file
