@@ -148,6 +148,8 @@ class AutoTrader():
         self.data_file  = None
         self.MTF_data_files = None
         self.local_data = None
+        self.local_quote_data = None
+        self.auxdata = None
         self.backtest_initial_balance = None
         self.backtest_spread = None
         self.backtest_commission = None
@@ -300,8 +302,11 @@ class AutoTrader():
         for strategy in self.strategies:
             for instrument in self.strategies[strategy]['WATCHLIST']:
                 data_dict = self.local_data[instrument] if self.local_data is not None else None
+                quote_data_dict = self.local_quote_data[instrument] if self.local_quote_data is not None else None
+                auxdata = self.auxdata[instrument] if self.auxdata is not None else None
                 bot = AutoTraderBot(instrument, self.strategies[strategy],
-                                    self.broker, data_dict, self)
+                                    self.broker, data_dict, quote_data_dict, 
+                                    auxdata, self)
                 
                 if self.detach_bot is True and self.backtest_mode is False:
                     # Send bot to bot manager to monitor stream
@@ -596,7 +601,8 @@ class AutoTrader():
         self.MTF_initialisation = MTF_initialisation
         self.jupyter_notebook = jupyter_notebook
         
-    def add_data(self, data_dict, data_directory='price_data', abs_dir_path=None):
+    def add_data(self, data_dict=None, quote_data=None, 
+                 data_directory='price_data', abs_dir_path=None, auxdata=None):
         ''' 
         Add local data to run backtest on. Note that to ensure proper directory 
         configuration, this method should only be called after calling 
@@ -618,7 +624,17 @@ class AutoTrader():
                  Note that the filenames in the dict above must be present in the 
                  data directory, as specified by the data_directory or abs_dir_path
                  parameters.
-                 
+                
+            quote_data (dict): a dictionary containing the quote data filenames 
+                of the datasets provided in data_dict. For example:
+                    quote_data = {'product1': 'product1_quote.csv',
+                                  'product2': 'product2_quote.csv'}
+                
+                In the case of MTF data, quote data should only be provided for
+                    the base timeframe (ie. the data which will be iterated on
+                    when backtesting). Therefore, the quote_data dict will look
+                    the same for single timeframe and MTF backtests.
+            
             data_directory (str): the name of the sub-directory containing price
                 data files. This directory should be located in the project
                 home directory (at.home_dir).
@@ -626,27 +642,61 @@ class AutoTrader():
             abs_dir_path: the absolute path to the data_directory. This parameter
                 may be used when the datafiles are stored outside of the project
                 directory.
+            
+            auxdata (dict): a dictionary containing raw data to supplement the 
+                data passed to the strategy module. For strategies involving 
+                multiple products, the keys of this dictionary must correspond
+                to the products, with the auxdata in nested dictionaries or 
+                otherwise. Examples include:
+                    auxdata = {'product1': aux_price_data,
+                               'product2': {'extra_data1': dataset1,
+                                            'extra_data2': dataset2}
+                               }
         '''
+        # TODO - add option to specify strategy, in case multiple strategies
+        # (requiring different data) are added to the instance
         
         dir_path = abs_dir_path if abs_dir_path is not None else os.path.join(self.home_dir, data_directory)
         
-        # Assign local data attribute
-        local_data = {}
+        # TODO - wrap the loops below into a signle for loop with dict assignment
         
-        # for loop to populate local_data
-        for product in data_dict:
-            if type(data_dict[product]) == dict:
-                # MTF data
-                MTF_data = {}
-                for timeframe in data_dict[product]:
-                    MTF_data[timeframe] = os.path.join(dir_path, data_dict[product][timeframe])
-                
-                local_data[product] = MTF_data
-            else:
-                # Single timeframe data
-                local_data[product] = os.path.join(dir_path, data_dict[product])
+        # Trading data
+        if data_dict is not None:
+            # Assign local data attribute
+            local_data = {}
+            
+            # Populate local_data
+            for product in data_dict:
+                if type(data_dict[product]) == dict:
+                    # MTF data
+                    MTF_data = {}
+                    for timeframe in data_dict[product]:
+                        MTF_data[timeframe] = os.path.join(dir_path, data_dict[product][timeframe])
+                    
+                    local_data[product] = MTF_data
+                else:
+                    # Single timeframe data
+                    local_data[product] = os.path.join(dir_path, data_dict[product])
+                    
+            self.local_data = local_data
         
-        self.local_data = local_data
+        # Quote data
+        if quote_data is not None:
+            # Assign local data attribute
+            local_quote_data = {}
+            
+            # Populate local_quote_data
+            for product in quote_data:
+                if type(quote_data[product]) == dict:
+                    raise Exception("Only a single quote-data file should be " +\
+                                    "provided per instrument traded.")
+                    
+                else:
+                    local_quote_data[product] = os.path.join(dir_path, quote_data[product])
+            
+            self.local_quote_data = local_quote_data
+        
+        self.auxdata = auxdata
     
     def scan(self, strategy_filename=None, strategy_dict=None, scan_index=None):
         '''
@@ -1314,7 +1364,10 @@ class AutoTrader():
         data = [bot.data for bot in self.bots_deployed]
         
         for i, dat in enumerate(data):
+            # Initialise common index
             comm_index = dat.index
+            
+            # Update common index by intersection with other data 
             for j, dat_2 in enumerate(data):
                 comm_index = comm_index.intersection(dat_2.index)
             
